'use strict';

/*!
 * sub-index | MIT (c) Shinnosuke Watanabe
 * https://github.com/shinnn/sub-index
*/
function subIndex(arr, index) {
  if (!Array.isArray(arr)) {
    throw new TypeError('Expected an array, but got ' + String(arr) + ' (' + typeof arr + ').');
  }

  if (arr.length === 0) {
    throw new TypeError('Expected a non-empty array, but got [].');
  }

  if (typeof index !== 'number') {
    throw new TypeError('Expected an index of array (integer), but got ' + String(index) + '(' + typeof index + ').');
  }

  var indexType = typeof index;

  if (indexType !== 'number') {
    throw new TypeError(
      'Expected an index of array, but got ' +
      String(index) +
      ' (' + typeof index + ').'
    );
  }

  if (!isFinite(index) || Math.floor(index) !== index) {
    throw new Error('Expected an index of array (integer), but got ' + String(index) + '.');
  }

  if (index < 0) {
    throw new Error('Expected an index of array (positive integer), but got ' + String(index) + '.');
  }

  var arrLength = arr.length;

  if (index >= arrLength) {
    throw new Error(
      'Expected an index of array, but got ' +
      String(index) +
      ' that exceeds the maximum index of the array (' + arrLength + ').');
  }

  if (arr.length === 1) {
    return null;
  }

  if (index === 0) {
    return 1;
  }

  return index - 1;
}

module.exports = subIndex;
